# Copyright (c) 2019, Trevor Highfill <trevor.highfill@outlook.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: diy
type: stdout
short_description: Customize the output
version_added: 0.2.0
description:
  - Callback plugin that allows you to supply your own custom callback templates to be output.
author: Trevor Highfill (@theque5t)
extends_documentation_fragment:
  - ansible.builtin.default_callback
notes:
  - Uses the P(ansible.builtin.default#callback) callback plugin output when a custom callback V(message(msg\)) is not provided.
  - Makes the callback event data available using the C(ansible_callback_diy) dictionary, which can be used in the templating
    context for the options. The dictionary is only available in the templating context for the options. It is not a variable
    that is available using the other various execution contexts, such as playbook, play, task, and so on so forth.
  - Options being set by their respective variable input can only be set using the variable if the variable was set in a context
    that is available to the respective callback. Use the C(ansible_callback_diy) dictionary to see what is available to a
    callback. Additionally, C(ansible_callback_diy.top_level_var_names) outputs the top level variable names available
    to the callback.
  - Each option value is rendered as a template before being evaluated. This allows for the dynamic usage of an option. For
    example, V("{{ 'yellow' if ansible_callback_diy.result.is_changed else 'bright green' }}").
  - 'B(Condition) for all C(msg) options: if value V(is None or omit), then the option is not being used. B(Effect): use of
    the C(default) callback plugin for output.'
  - 'B(Condition) for all C(msg) options: if value V(is not None and not omit and length is not greater than 0), then the
    option is being used without output. B(Effect): suppress output.'
  - 'B(Condition) for all C(msg) options: if value V(is not None and not omit and length is greater than 0), then the option
    is being used with output. B(Effect): render value as template and output.'
  - 'Valid color values: V(black), V(bright gray), V(blue), V(white), V(green), V(bright blue), V(cyan), V(bright green),
    V(red), V(bright cyan), V(purple), V(bright red), V(yellow), V(bright purple), V(dark gray), V(bright yellow), V(magenta),
    V(bright magenta), V(normal).'
seealso:
  - name: default – default Ansible screen output
    description: The official documentation on the B(default) callback plugin.
    link: https://docs.ansible.com/projects/ansible/latest/plugins/callback/default.html
requirements:
  - set as stdout_callback in configuration
options:
  on_any_msg:
    description: Output to be used for callback on_any.
    ini:
      - section: callback_diy
        key: on_any_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_ON_ANY_MSG
    vars:
      - name: ansible_callback_diy_on_any_msg
    type: str

  on_any_msg_color:
    description:
      - Output color to be used for O(on_any_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: on_any_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_ON_ANY_MSG_COLOR
    vars:
      - name: ansible_callback_diy_on_any_msg_color
    type: str

  runner_on_failed_msg:
    description: Output to be used for callback runner_on_failed.
    ini:
      - section: callback_diy
        key: runner_on_failed_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_FAILED_MSG
    vars:
      - name: ansible_callback_diy_runner_on_failed_msg
    type: str

  runner_on_failed_msg_color:
    description:
      - Output color to be used for O(runner_on_failed_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_failed_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_FAILED_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_failed_msg_color
    type: str

  runner_on_ok_msg:
    description: Output to be used for callback runner_on_ok.
    ini:
      - section: callback_diy
        key: runner_on_ok_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_OK_MSG
    vars:
      - name: ansible_callback_diy_runner_on_ok_msg
    type: str

  runner_on_ok_msg_color:
    description:
      - Output color to be used for O(runner_on_ok_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_ok_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_OK_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_ok_msg_color
    type: str

  runner_on_skipped_msg:
    description: Output to be used for callback runner_on_skipped.
    ini:
      - section: callback_diy
        key: runner_on_skipped_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_SKIPPED_MSG
    vars:
      - name: ansible_callback_diy_runner_on_skipped_msg
    type: str

  runner_on_skipped_msg_color:
    description:
      - Output color to be used for O(runner_on_skipped_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_skipped_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_SKIPPED_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_skipped_msg_color
    type: str

  runner_on_unreachable_msg:
    description: Output to be used for callback runner_on_unreachable.
    ini:
      - section: callback_diy
        key: runner_on_unreachable_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_UNREACHABLE_MSG
    vars:
      - name: ansible_callback_diy_runner_on_unreachable_msg
    type: str

  runner_on_unreachable_msg_color:
    description:
      - Output color to be used for O(runner_on_unreachable_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_unreachable_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_UNREACHABLE_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_unreachable_msg_color
    type: str

  playbook_on_start_msg:
    description: Output to be used for callback playbook_on_start.
    ini:
      - section: callback_diy
        key: playbook_on_start_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_START_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_start_msg
    type: str

  playbook_on_start_msg_color:
    description:
      - Output color to be used for O(playbook_on_start_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_start_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_START_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_start_msg_color
    type: str

  playbook_on_notify_msg:
    description: Output to be used for callback playbook_on_notify.
    ini:
      - section: callback_diy
        key: playbook_on_notify_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NOTIFY_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_notify_msg
    type: str

  playbook_on_notify_msg_color:
    description:
      - Output color to be used for O(playbook_on_notify_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_notify_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NOTIFY_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_notify_msg_color
    type: str

  playbook_on_no_hosts_matched_msg:
    description: Output to be used for callback playbook_on_no_hosts_matched.
    ini:
      - section: callback_diy
        key: playbook_on_no_hosts_matched_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_MATCHED_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_no_hosts_matched_msg
    type: str

  playbook_on_no_hosts_matched_msg_color:
    description:
      - Output color to be used for O(playbook_on_no_hosts_matched_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_no_hosts_matched_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_MATCHED_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_no_hosts_matched_msg_color
    type: str

  playbook_on_no_hosts_remaining_msg:
    description: Output to be used for callback playbook_on_no_hosts_remaining.
    ini:
      - section: callback_diy
        key: playbook_on_no_hosts_remaining_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_REMAINING_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_no_hosts_remaining_msg
    type: str

  playbook_on_no_hosts_remaining_msg_color:
    description:
      - Output color to be used for O(playbook_on_no_hosts_remaining_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_no_hosts_remaining_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_NO_HOSTS_REMAINING_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_no_hosts_remaining_msg_color
    type: str

  playbook_on_task_start_msg:
    description: Output to be used for callback playbook_on_task_start.
    ini:
      - section: callback_diy
        key: playbook_on_task_start_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_TASK_START_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_task_start_msg
    type: str

  playbook_on_task_start_msg_color:
    description:
      - Output color to be used for O(playbook_on_task_start_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_task_start_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_TASK_START_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_task_start_msg_color
    type: str

  playbook_on_handler_task_start_msg:
    description: Output to be used for callback playbook_on_handler_task_start.
    ini:
      - section: callback_diy
        key: playbook_on_handler_task_start_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_HANDLER_TASK_START_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_handler_task_start_msg
    type: str

  playbook_on_handler_task_start_msg_color:
    description:
      - Output color to be used for O(playbook_on_handler_task_start_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_handler_task_start_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_HANDLER_TASK_START_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_handler_task_start_msg_color
    type: str

  playbook_on_vars_prompt_msg:
    description: Output to be used for callback playbook_on_vars_prompt.
    ini:
      - section: callback_diy
        key: playbook_on_vars_prompt_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_VARS_PROMPT_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_vars_prompt_msg
    type: str

  playbook_on_vars_prompt_msg_color:
    description:
      - Output color to be used for O(playbook_on_vars_prompt_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_vars_prompt_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_VARS_PROMPT_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_vars_prompt_msg_color
    type: str

  playbook_on_play_start_msg:
    description: Output to be used for callback playbook_on_play_start.
    ini:
      - section: callback_diy
        key: playbook_on_play_start_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_PLAY_START_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_play_start_msg
    type: str

  playbook_on_play_start_msg_color:
    description:
      - Output color to be used for O(playbook_on_play_start_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_play_start_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_PLAY_START_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_play_start_msg_color
    type: str

  playbook_on_stats_msg:
    description: Output to be used for callback playbook_on_stats.
    ini:
      - section: callback_diy
        key: playbook_on_stats_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_STATS_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_stats_msg
    type: str

  playbook_on_stats_msg_color:
    description:
      - Output color to be used for O(playbook_on_stats_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_stats_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_STATS_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_stats_msg_color
    type: str

  on_file_diff_msg:
    description: Output to be used for callback on_file_diff.
    ini:
      - section: callback_diy
        key: on_file_diff_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_ON_FILE_DIFF_MSG
    vars:
      - name: ansible_callback_diy_on_file_diff_msg
    type: str

  on_file_diff_msg_color:
    description:
      - Output color to be used for O(on_file_diff_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: on_file_diff_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_ON_FILE_DIFF_MSG_COLOR
    vars:
      - name: ansible_callback_diy_on_file_diff_msg_color
    type: str

  playbook_on_include_msg:
    description: Output to be used for callback playbook_on_include.
    ini:
      - section: callback_diy
        key: playbook_on_include_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_INCLUDE_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_include_msg
    type: str

  playbook_on_include_msg_color:
    description:
      - Output color to be used for O(playbook_on_include_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_include_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_INCLUDE_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_include_msg_color
    type: str

  runner_item_on_ok_msg:
    description: Output to be used for callback runner_item_on_ok.
    ini:
      - section: callback_diy
        key: runner_item_on_ok_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_OK_MSG
    vars:
      - name: ansible_callback_diy_runner_item_on_ok_msg
    type: str

  runner_item_on_ok_msg_color:
    description:
      - Output color to be used for O(runner_item_on_ok_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_item_on_ok_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_OK_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_item_on_ok_msg_color
    type: str

  runner_item_on_failed_msg:
    description: Output to be used for callback runner_item_on_failed.
    ini:
      - section: callback_diy
        key: runner_item_on_failed_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_FAILED_MSG
    vars:
      - name: ansible_callback_diy_runner_item_on_failed_msg
    type: str

  runner_item_on_failed_msg_color:
    description:
      - Output color to be used for O(runner_item_on_failed_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_item_on_failed_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_FAILED_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_item_on_failed_msg_color
    type: str

  runner_item_on_skipped_msg:
    description: Output to be used for callback runner_item_on_skipped.
    ini:
      - section: callback_diy
        key: runner_item_on_skipped_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_SKIPPED_MSG
    vars:
      - name: ansible_callback_diy_runner_item_on_skipped_msg
    type: str

  runner_item_on_skipped_msg_color:
    description:
      - Output color to be used for O(runner_item_on_skipped_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_item_on_skipped_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ITEM_ON_SKIPPED_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_item_on_skipped_msg_color
    type: str

  runner_retry_msg:
    description: Output to be used for callback runner_retry.
    ini:
      - section: callback_diy
        key: runner_retry_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_RETRY_MSG
    vars:
      - name: ansible_callback_diy_runner_retry_msg
    type: str

  runner_retry_msg_color:
    description:
      - Output color to be used for O(runner_retry_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_retry_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_RETRY_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_retry_msg_color
    type: str

  runner_on_start_msg:
    description: Output to be used for callback runner_on_start.
    ini:
      - section: callback_diy
        key: runner_on_start_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_START_MSG
    vars:
      - name: ansible_callback_diy_runner_on_start_msg
    type: str

  runner_on_start_msg_color:
    description:
      - Output color to be used for O(runner_on_start_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_start_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_START_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_start_msg_color
    type: str

  runner_on_no_hosts_msg:
    description: Output to be used for callback runner_on_no_hosts.
    ini:
      - section: callback_diy
        key: runner_on_no_hosts_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_NO_HOSTS_MSG
    vars:
      - name: ansible_callback_diy_runner_on_no_hosts_msg
    type: str

  runner_on_no_hosts_msg_color:
    description:
      - Output color to be used for O(runner_on_no_hosts_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: runner_on_no_hosts_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_RUNNER_ON_NO_HOSTS_MSG_COLOR
    vars:
      - name: ansible_callback_diy_runner_on_no_hosts_msg_color
    type: str

  playbook_on_setup_msg:
    description: Output to be used for callback playbook_on_setup.
    ini:
      - section: callback_diy
        key: playbook_on_setup_msg
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_SETUP_MSG
    vars:
      - name: ansible_callback_diy_playbook_on_setup_msg
    type: str

  playbook_on_setup_msg_color:
    description:
      - Output color to be used for O(playbook_on_setup_msg).
      - Template should render a L(valid color value,#notes).
    ini:
      - section: callback_diy
        key: playbook_on_setup_msg_color
    env:
      - name: ANSIBLE_CALLBACK_DIY_PLAYBOOK_ON_SETUP_MSG_COLOR
    vars:
      - name: ansible_callback_diy_playbook_on_setup_msg_color
    type: str
"""

EXAMPLES = r"""
ansible.cfg: >
  # Enable plugin
  [defaults]
  stdout_callback=community.general.diy

  [callback_diy]
  # Output when playbook starts
  playbook_on_start_msg="DIY output(via ansible.cfg): playbook example: {{ ansible_callback_diy.playbook.file_name }}"
  playbook_on_start_msg_color=yellow

  # Comment out to allow default plugin output
  # playbook_on_play_start_msg="PLAY: starting play {{ ansible_callback_diy.play.name }}"

  # Accept on_skipped_msg or ansible_callback_diy_runner_on_skipped_msg as input vars
  # If neither are supplied, omit the option
  runner_on_skipped_msg="{{ on_skipped_msg | default(ansible_callback_diy_runner_on_skipped_msg) | default(omit) }}"

  # Newline after every callback
  # on_any_msg='{{ " " | join("\n") }}'

playbook.yml: >-
  ---
  - name: "Default plugin output: play example"
    hosts: localhost
    gather_facts: false
    tasks:
      - name:  Default plugin output
        ansible.builtin.debug:
          msg: default plugin output

  - name: Override from play vars
    hosts: localhost
    gather_facts: false
    vars:
      ansible_connection: local
      green: "\e[0m\e[38;5;82m"
      yellow: "\e[0m\e[38;5;11m"
      bright_purple: "\e[0m\e[38;5;105m"
      cyan: "\e[0m\e[38;5;51m"
      green_bg_black_fg: "\e[0m\e[48;5;40m\e[38;5;232m"
      yellow_bg_black_fg: "\e[0m\e[48;5;226m\e[38;5;232m"
      purple_bg_white_fg: "\e[0m\e[48;5;57m\e[38;5;255m"
      cyan_bg_black_fg: "\e[0m\e[48;5;87m\e[38;5;232m"
      magenta: "\e[38;5;198m"
      white: "\e[0m\e[38;5;255m"
      ansible_callback_diy_playbook_on_play_start_msg: "\n{{green}}DIY output(via play vars): play example: {{magenta}}{{ansible_callback_diy.play.name}}\n\n"
      ansible_callback_diy_playbook_on_task_start_msg: "DIY output(via play vars): task example: {{ ansible_callback_diy.task.name }}"
      ansible_callback_diy_playbook_on_task_start_msg_color: cyan
      ansible_callback_diy_playbook_on_stats_msg: |+2
                CUSTOM STATS
        ==============================
        {% for key in ansible_callback_diy.stats | sort %}
        {% if ansible_callback_diy.stats[key] %}
        {% if key == 'ok' %}
        {% set color_one = lookup('vars','green_bg_black_fg') %}
        {% set prefix = '      ' %}
        {% set suffix = '     ' %}
        {% set color_two = lookup('vars','green') %}
        {% elif key == 'changed' %}
        {% set color_one = lookup('vars','yellow_bg_black_fg') %}
        {% set prefix = '   ' %}
        {% set suffix = '   ' %}
        {% set color_two = lookup('vars','yellow') %}
        {% elif key == 'processed' %}
        {% set color_one = lookup('vars','purple_bg_white_fg') %}
        {% set prefix = '  ' %}
        {% set suffix = '  ' %}
        {% set color_two = lookup('vars','bright_purple') %}
        {% elif key == 'skipped' %}
        {% set color_one = lookup('vars','cyan_bg_black_fg') %}
        {% set prefix = '   ' %}
        {% set suffix = '   ' %}
        {% set color_two = lookup('vars','cyan') %}
        {% else %}
        {% set color_one = "" %}
        {% set prefix = "" %}
        {% set suffix = "" %}
        {% set color_two = "" %}
        {% endif %}
        {{ color_one }}{{ "%s%s%s" | format(prefix,key,suffix) }}{{ color_two }}: {{ ansible_callback_diy.stats[key] | to_nice_yaml }}
        {% endif %}
        {% endfor %}

    tasks:
      - name: Custom banner with default plugin result output
        ansible.builtin.debug:
          msg: "default plugin output: result example"

      - name: Override from task vars
        ansible.builtin.debug:
          msg: "example {{ two }}"
        changed_when: true
        vars:
          white_fg_red_bg: "\e[0m\e[48;5;1m"
          two: "{{ white_fg_red_bg }}    2    "
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_playbook_on_task_start_msg_color: bright magenta
          ansible_callback_diy_runner_on_ok_msg: "DIY output(via task vars): result example: \n{{ ansible_callback_diy.result.output.msg }}\n"
          ansible_callback_diy_runner_on_ok_msg_color: "{{ 'yellow' if ansible_callback_diy.result.is_changed else 'bright green' }}"

      - name: Suppress output
        ansible.builtin.debug:
          msg: i should not be displayed
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: ""
          ansible_callback_diy_runner_on_ok_msg: ""

      - name: Using alias vars (see ansible.cfg)
        ansible.builtin.debug:
          msg:
        when: false
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: ""
          on_skipped_msg: "DIY output(via task vars): skipped example:\n\e[0m\e[38;5;4m\u25b6\u25b6 {{ ansible_callback_diy.result.task.name }}\n"
          on_skipped_msg_color: white

      - name: Just stdout
        ansible.builtin.command: echo some stdout
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\n"
          ansible_callback_diy_runner_on_ok_msg: "{{ ansible_callback_diy.result.output.stdout }}\n"

      - name: Multiline output
        ansible.builtin.debug:
          msg: "{{ multiline }}"
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          multiline: "line\nline\nline"
          ansible_callback_diy_runner_on_ok_msg: |+2
            some
            {{ ansible_callback_diy.result.output.msg }}
            output

          ansible_callback_diy_playbook_on_task_start_msg_color: bright blue

      - name: Indentation
        ansible.builtin.debug:
          msg: "{{ item.msg }}"
        with_items:
          - { indent: 1, msg: one., color: red }
          - { indent: 2, msg: two.., color: yellow }
          - { indent: 3, msg: three..., color: bright yellow }
        vars:
          ansible_callback_diy_runner_item_on_ok_msg: "{{ ansible_callback_diy.result.output.msg | indent(item.indent, True) }}"
          ansible_callback_diy_runner_item_on_ok_msg_color: "{{ item.color }}"
          ansible_callback_diy_runner_on_ok_msg: "GO!!!"
          ansible_callback_diy_runner_on_ok_msg_color: bright green

      - name: Using lookup and template as file
        ansible.builtin.shell: "echo {% raw %}'output from {{ file_name }}'{% endraw %} > {{ file_name }}"
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          file_name: diy_file_template_example
          ansible_callback_diy_runner_on_ok_msg: "{{ lookup('template', file_name) }}"

      - name: 'Look at top level vars available to the "runner_on_ok" callback'
        ansible.builtin.debug:
          msg: ''
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_runner_on_ok_msg: |+2
            {% for var in (ansible_callback_diy.top_level_var_names|reject('match','vars|ansible_callback_diy.*')) | sort %}
            {{ green }}{{ var }}:
              {{ white }}{{ lookup('vars', var) }}

            {% endfor %}
          ansible_callback_diy_runner_on_ok_msg_color: white

      - name: 'Look at event data available to the "runner_on_ok" callback'
        ansible.builtin.debug:
          msg: ''
        vars:
          ansible_callback_diy_playbook_on_task_start_msg: "\nDIY output(via task vars): task example: {{ ansible_callback_diy.task.name }}"
          ansible_callback_diy_runner_on_ok_msg: |+2
            {% for key in ansible_callback_diy | sort %}
            {{ green }}{{ key }}:
              {{ white }}{{ ansible_callback_diy[key] }}

            {% endfor %}
"""

import sys
from contextlib import contextmanager

from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.callback.default import CallbackModule as Default
from ansible.template import Templar
from ansible.vars.manager import VariableManager

try:
    from ansible.template import trust_as_template  # noqa: F401, pylint: disable=unused-import

    SUPPORTS_DATA_TAGGING = True
except ImportError:
    SUPPORTS_DATA_TAGGING = False


class DummyStdout:
    def flush(self):
        pass

    def write(self, b):
        pass

    def writelines(self, l):
        pass


class CallbackModule(Default):
    """
    Callback plugin that allows you to supply your own custom callback templates to be output.
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "stdout"
    CALLBACK_NAME = "community.general.diy"

    DIY_NS = "ansible_callback_diy"

    @contextmanager
    def _suppress_stdout(self, enabled):
        saved_stdout = sys.stdout
        if enabled:
            sys.stdout = DummyStdout()
        yield
        sys.stdout = saved_stdout

    def _get_output_specification(self, loader, variables):
        _ret = {}
        _calling_method = sys._getframe(1).f_code.co_name
        _callback_type = _calling_method[3:] if _calling_method[:3] == "v2_" else _calling_method
        _callback_options = ["msg", "msg_color"]

        for option in _callback_options:
            _option_name = f"{_callback_type}_{option}"
            _option_template = variables.get(f"{self.DIY_NS}_{_option_name}", self.get_option(_option_name))
            _ret.update({option: self._template(loader=loader, template=_option_template, variables=variables)})

        _ret.update({"vars": variables})

        return _ret

    def _using_diy(self, spec):
        sentinel = object()
        omit = spec["vars"].get("omit", sentinel)
        # With Data Tagging, omit is sentinel
        return (spec["msg"] is not None) and (spec["msg"] != omit or omit is sentinel)

    def _parent_has_callback(self):
        return hasattr(super(), sys._getframe(1).f_code.co_name)

    def _template(self, loader, template, variables):
        _templar = Templar(loader=loader, variables=variables)
        return _templar.template(template, preserve_trailing_newlines=True, convert_data=False, escape_backslashes=True)

    def _output(self, spec, stderr=False):
        _msg = to_text(spec["msg"])
        if len(_msg) > 0:
            self._display.display(msg=_msg, color=spec["msg_color"], stderr=stderr)

    def _get_vars(
        self,
        playbook,
        play=None,
        host=None,
        task=None,
        included_file=None,
        handler=None,
        result=None,
        stats=None,
        remove_attr_ref_loop=True,
    ):
        def _get_value(obj, attr=None, method=None):
            if attr:
                return getattr(obj, attr, getattr(obj, f"_{attr}", None))

            if method:
                _method = getattr(obj, method)
                return _method()

        def _remove_attr_ref_loop(obj, attributes):
            _loop_var = getattr(obj, "loop_control", None)
            _loop_var = _loop_var or "item"

            for attr in attributes:
                if str(_loop_var) in str(_get_value(obj=obj, attr=attr)):
                    attributes.remove(attr)

            return attributes

        class CallbackDIYDict(dict):
            def __deepcopy__(self, memo):
                return self

        _ret = {}

        _variable_manager = VariableManager(loader=playbook.get_loader())

        _all = _variable_manager.get_vars()
        if play:
            _all = play.get_variable_manager().get_vars(
                play=play, host=(host if host else getattr(result, "_host", None)), task=(handler if handler else task)
            )
        _ret.update(_all)

        _ret.update(_ret.get(self.DIY_NS, {self.DIY_NS: {} if SUPPORTS_DATA_TAGGING else CallbackDIYDict()}))

        _ret[self.DIY_NS].update({"playbook": {}})
        _playbook_attributes = ["entries", "file_name", "basedir"]

        for attr in _playbook_attributes:
            _ret[self.DIY_NS]["playbook"].update({attr: _get_value(obj=playbook, attr=attr)})

        if play:
            _ret[self.DIY_NS].update({"play": {}})
            _play_attributes = [
                "any_errors_fatal",
                "become",
                "become_flags",
                "become_method",
                "become_user",
                "check_mode",
                "collections",
                "connection",
                "debugger",
                "diff",
                "environment",
                "fact_path",
                "finalized",
                "force_handlers",
                "gather_facts",
                "gather_subset",
                "gather_timeout",
                "handlers",
                "hosts",
                "ignore_errors",
                "ignore_unreachable",
                "included_conditional",
                "included_path",
                "max_fail_percentage",
                "module_defaults",
                "name",
                "no_log",
                "only_tags",
                "order",
                "port",
                "post_tasks",
                "pre_tasks",
                "remote_user",
                "removed_hosts",
                "roles",
                "run_once",
                "serial",
                "skip_tags",
                "squashed",
                "strategy",
                "tags",
                "tasks",
                "uuid",
                "validated",
                "vars_files",
                "vars_prompt",
            ]

            for attr in _play_attributes:
                _ret[self.DIY_NS]["play"].update({attr: _get_value(obj=play, attr=attr)})

        if host:
            _ret[self.DIY_NS].update({"host": {}})
            _host_attributes = ["name", "uuid", "address", "implicit"]

            for attr in _host_attributes:
                _ret[self.DIY_NS]["host"].update({attr: _get_value(obj=host, attr=attr)})

        if task:
            _ret[self.DIY_NS].update({"task": {}})
            _task_attributes = [
                "action",
                "any_errors_fatal",
                "args",
                "async",
                "async_val",
                "become",
                "become_flags",
                "become_method",
                "become_user",
                "changed_when",
                "check_mode",
                "collections",
                "connection",
                "debugger",
                "delay",
                "delegate_facts",
                "delegate_to",
                "diff",
                "environment",
                "failed_when",
                "finalized",
                "ignore_errors",
                "ignore_unreachable",
                "loop",
                "loop_control",
                "loop_with",
                "module_defaults",
                "name",
                "no_log",
                "notify",
                "parent",
                "poll",
                "port",
                "register",
                "remote_user",
                "retries",
                "role",
                "run_once",
                "squashed",
                "tags",
                "untagged",
                "until",
                "uuid",
                "validated",
                "when",
            ]

            # remove arguments that reference a loop var because they cause templating issues in
            # callbacks that do not have the loop context(e.g. playbook_on_task_start)
            if task.loop and remove_attr_ref_loop:
                _task_attributes = _remove_attr_ref_loop(obj=task, attributes=_task_attributes)

            for attr in _task_attributes:
                _ret[self.DIY_NS]["task"].update({attr: _get_value(obj=task, attr=attr)})

        if included_file:
            _ret[self.DIY_NS].update({"included_file": {}})
            _included_file_attributes = ["args", "filename", "hosts", "is_role", "task"]

            for attr in _included_file_attributes:
                _ret[self.DIY_NS]["included_file"].update({attr: _get_value(obj=included_file, attr=attr)})

        if handler:
            _ret[self.DIY_NS].update({"handler": {}})
            _handler_attributes = [
                "action",
                "any_errors_fatal",
                "args",
                "async",
                "async_val",
                "become",
                "become_flags",
                "become_method",
                "become_user",
                "changed_when",
                "check_mode",
                "collections",
                "connection",
                "debugger",
                "delay",
                "delegate_facts",
                "delegate_to",
                "diff",
                "environment",
                "failed_when",
                "finalized",
                "ignore_errors",
                "ignore_unreachable",
                "listen",
                "loop",
                "loop_control",
                "loop_with",
                "module_defaults",
                "name",
                "no_log",
                "notified_hosts",
                "notify",
                "parent",
                "poll",
                "port",
                "register",
                "remote_user",
                "retries",
                "role",
                "run_once",
                "squashed",
                "tags",
                "untagged",
                "until",
                "uuid",
                "validated",
                "when",
            ]

            if handler.loop and remove_attr_ref_loop:
                _handler_attributes = _remove_attr_ref_loop(obj=handler, attributes=_handler_attributes)

            for attr in _handler_attributes:
                _ret[self.DIY_NS]["handler"].update({attr: _get_value(obj=handler, attr=attr)})

            _ret[self.DIY_NS]["handler"].update({"is_host_notified": handler.is_host_notified(host)})

        if result:
            _ret[self.DIY_NS].update({"result": {}})
            _result_attributes = ["host", "task", "task_name"]

            for attr in _result_attributes:
                _ret[self.DIY_NS]["result"].update({attr: _get_value(obj=result, attr=attr)})

            _result_methods = ["is_changed", "is_failed", "is_skipped", "is_unreachable"]

            for method in _result_methods:
                _ret[self.DIY_NS]["result"].update({method: _get_value(obj=result, method=method)})

            _ret[self.DIY_NS]["result"].update({"output": getattr(result, "_result", None)})

            _ret.update(result._result)

        if stats:
            _ret[self.DIY_NS].update({"stats": {}})
            _stats_attributes = [
                "changed",
                "custom",
                "dark",
                "failures",
                "ignored",
                "ok",
                "processed",
                "rescued",
                "skipped",
            ]

            for attr in _stats_attributes:
                _ret[self.DIY_NS]["stats"].update({attr: _get_value(obj=stats, attr=attr)})

        _ret[self.DIY_NS].update({"top_level_var_names": list(_ret.keys())})

        return _ret

    def v2_on_any(self, *args, **kwargs):
        self._diy_spec = self._get_output_specification(loader=self._diy_loader, variables=self._diy_spec["vars"])

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_on_any(*args, **kwargs)

    def v2_runner_on_failed(self, result, ignore_errors=False):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec, stderr=(not ignore_errors))

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_on_failed(result, ignore_errors)

    def v2_runner_on_ok(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_on_ok(result)

    def v2_runner_on_skipped(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_on_skipped(result)

    def v2_runner_on_unreachable(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_on_unreachable(result)

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_poll(self, result):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_ok(self, result):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_runner_on_async_failed(self, result):
        pass

    def v2_runner_item_on_ok(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False,
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_item_on_ok(result)

    def v2_runner_item_on_failed(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False,
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_item_on_failed(result)

    def v2_runner_item_on_skipped(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_task,
                result=result,
                remove_attr_ref_loop=False,
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_item_on_skipped(result)

    def v2_runner_retry(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_retry(result)

    def v2_runner_on_start(self, host, task):
        self._diy_host = host
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, host=self._diy_host, task=self._diy_task
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_runner_on_start(host, task)

    def v2_playbook_on_start(self, playbook):
        self._diy_playbook = playbook
        self._diy_loader = self._diy_playbook.get_loader()

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader, variables=self._get_vars(playbook=self._diy_playbook)
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_start(playbook)

    def v2_playbook_on_notify(self, handler, host):
        self._diy_handler = handler
        self._diy_host = host

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, host=self._diy_host, handler=self._diy_handler
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_notify(handler, host)

    def v2_playbook_on_no_hosts_matched(self):
        self._diy_spec = self._get_output_specification(loader=self._diy_loader, variables=self._diy_spec["vars"])

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_no_hosts_matched()

    def v2_playbook_on_no_hosts_remaining(self):
        self._diy_spec = self._get_output_specification(loader=self._diy_loader, variables=self._diy_spec["vars"])

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_no_hosts_remaining()

    def v2_playbook_on_task_start(self, task, is_conditional):
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_task_start(task, is_conditional)

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_cleanup_task_start(self, task):
        pass

    def v2_playbook_on_handler_task_start(self, task):
        self._diy_task = task

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_handler_task_start(task)

    def v2_playbook_on_vars_prompt(
        self,
        varname,
        private=True,
        prompt=None,
        encrypt=None,
        confirm=False,
        salt_size=None,
        salt=None,
        default=None,
        unsafe=None,
    ):
        self._diy_spec = self._get_output_specification(loader=self._diy_loader, variables=self._diy_spec["vars"])

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_vars_prompt(
                    varname, private, prompt, encrypt, confirm, salt_size, salt, default, unsafe
                )

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_import_for_host(self, result, imported_file):
        pass

    # not implemented as the call to this is not implemented yet
    def v2_playbook_on_not_import_for_host(self, result, missing_file):
        pass

    def v2_playbook_on_play_start(self, play):
        self._diy_play = play

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader, variables=self._get_vars(playbook=self._diy_playbook, play=self._diy_play)
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_play_start(play)

    def v2_playbook_on_stats(self, stats):
        self._diy_stats = stats

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(playbook=self._diy_playbook, play=self._diy_play, stats=self._diy_stats),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_stats(stats)

    def v2_playbook_on_include(self, included_file):
        self._diy_included_file = included_file

        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook,
                play=self._diy_play,
                task=self._diy_included_file._task,
                included_file=self._diy_included_file,
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_playbook_on_include(included_file)

    def v2_on_file_diff(self, result):
        self._diy_spec = self._get_output_specification(
            loader=self._diy_loader,
            variables=self._get_vars(
                playbook=self._diy_playbook, play=self._diy_play, task=self._diy_task, result=result
            ),
        )

        if self._using_diy(spec=self._diy_spec):
            self._output(spec=self._diy_spec)

        if self._parent_has_callback():
            with self._suppress_stdout(enabled=self._using_diy(spec=self._diy_spec)):
                super().v2_on_file_diff(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: iam_server_certificate_info
version_added: 1.0.0
short_description: Retrieve the information of a server certificate
description:
  - Retrieve the attributes of a server certificate.
author:
  - "Allen Sanabria (@linuxdynasty)"
options:
  name:
    description:
      - The name of the server certificate you are retrieving attributes for.
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Retrieve server certificate
  community.aws.iam_server_certificate_info:
    name: production-cert
  register: server_cert

- name: Fail if the server certificate name was not found
  community.aws.iam_server_certificate_info:
    name: production-cert
  register: server_cert
  failed_when: "{{ server_cert.results | length == 0 }}"
"""

RETURN = r"""
server_certificate_id:
    description: The 21 character certificate id
    returned: success
    type: str
    sample: "ADWAJXWTZAXIPIMQHMJPO"
certificate_body:
    description: The asn1der encoded PEM string
    returned: success
    type: str
    sample: "-----BEGIN CERTIFICATE-----\nbunch of random data\n-----END CERTIFICATE-----"
server_certificate_name:
    description: The name of the server certificate
    returned: success
    type: str
    sample: "server-cert-name"
arn:
    description: The Amazon resource name of the server certificate
    returned: success
    type: str
    sample: "arn:aws:iam::123456789012:server-certificate/server-cert-name"
path:
    description: The path of the server certificate
    returned: success
    type: str
    sample: "/"
expiration:
    description: The date and time this server certificate will expire, in ISO 8601 format.
    returned: success
    type: str
    sample: "2017-06-15T12:00:00+00:00"
upload_date:
    description: The date and time this server certificate was uploaded, in ISO 8601 format.
    returned: success
    type: str
    sample: "2015-04-25T00:36:40+00:00"
"""


try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_server_certs(iam, name=None):
    """Retrieve the attributes of a server certificate if it exists or all certs.
    Args:
        iam (botocore.client.IAM): The boto3 iam instance.

    Kwargs:
        name (str): The name of the server certificate.

    Basic Usage:
        >>> import boto3
        >>> iam = boto3.client('iam')
        >>> name = "server-cert-name"
        >>> results = get_server_certs(iam, name)
        {
            "upload_date": "2015-04-25T00:36:40+00:00",
            "server_certificate_id": "ADWAJXWTZAXIPIMQHMJPO",
            "certificate_body": "-----BEGIN CERTIFICATE-----\nbunch of random data\n-----END CERTIFICATE-----",
            "server_certificate_name": "server-cert-name",
            "expiration": "2017-06-15T12:00:00+00:00",
            "path": "/",
            "arn": "arn:aws:iam::123456789012:server-certificate/server-cert-name"
        }
    """
    results = dict()
    try:
        if name:
            server_certs = [iam.get_server_certificate(ServerCertificateName=name)["ServerCertificate"]]
        else:
            server_certs = iam.list_server_certificates()["ServerCertificateMetadataList"]

        for server_cert in server_certs:
            if not name:
                server_cert = iam.get_server_certificate(ServerCertificateName=server_cert["ServerCertificateName"])[
                    "ServerCertificate"
                ]
            cert_md = server_cert["ServerCertificateMetadata"]
            results[cert_md["ServerCertificateName"]] = {
                "certificate_body": server_cert["CertificateBody"],
                "server_certificate_id": cert_md["ServerCertificateId"],
                "server_certificate_name": cert_md["ServerCertificateName"],
                "arn": cert_md["Arn"],
                "path": cert_md["Path"],
                "expiration": cert_md["Expiration"].isoformat(),
                "upload_date": cert_md["UploadDate"].isoformat(),
            }

    except botocore.exceptions.ClientError:
        pass

    return results


def main():
    argument_spec = dict(
        name=dict(type="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        iam = module.client("iam")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    cert_name = module.params.get("name")
    results = get_server_certs(iam, cert_name)
    module.exit_json(results=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Steve Gargan <steve.gargan@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_session
short_description: Manipulate Consul sessions
description:
  - Allows the addition, modification and deletion of sessions in a Consul cluster. These sessions can then be used in conjunction
    with key value pairs to implement distributed locks. In depth documentation for working with sessions can be found at
    U(http://www.consul.io/docs/internals/sessions.html).
author:
  - Steve Gargan (@sgargan)
  - Håkon Lerring (@Hakon)
extends_documentation_fragment:
  - community.general._consul
  - community.general._consul.actiongroup_consul
  - community.general._consul.token
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  action_group:
    version_added: 8.3.0
options:
  id:
    description:
      - ID of the session, required when O(state) is either V(info) or V(remove).
    type: str
  state:
    description:
      - Whether the session should be present, in other words it should be created if it does not exist, or absent, removed
        if present. If created, the O(id) for the session is returned in the output. If V(absent), O(id) is required to remove
        the session. Info for a single session, all the sessions for a node or all available sessions can be retrieved by
        specifying V(info), V(node) or V(list) for the O(state); for V(node) or V(info), the node O(name) or session O(id)
        is required as parameter.
    choices: [absent, info, list, node, present]
    type: str
    default: present
  name:
    description:
      - The name that should be associated with the session. Required when O(state=node) is used.
    type: str
  delay:
    description:
      - The optional lock delay that can be attached to the session when it is created. Locks for invalidated sessions ar
        blocked from being acquired until this delay has expired. Durations are in seconds.
    type: int
    default: 15
  node:
    description:
      - The name of the node that with which the session is associated. By default this is the name of the agent.
    type: str
  datacenter:
    description:
      - The name of the datacenter in which the session exists or should be created.
    type: str
  checks:
    description:
      - Checks that are used to verify the session health. If all the checks fail, the session is invalidated and any locks
        associated with the session are released and can be acquired once the associated lock delay has expired.
    type: list
    elements: str
  behavior:
    description:
      - The optional behavior that can be attached to the session when it is created. This controls the behavior when a session
        is invalidated.
    choices: [delete, release]
    type: str
    default: release
  ttl:
    description:
      - Specifies the duration of a session in seconds (between 10 and 86400).
    type: int
    version_added: 5.4.0
  token:
    version_added: 5.6.0
"""

EXAMPLES = r"""
- name: Register basic session with Consul
  community.general.consul_session:
    name: session1

- name: Register a session with an existing check
  community.general.consul_session:
    name: session_with_check
    checks:
      - existing_check_name

- name: Register a session with lock_delay
  community.general.consul_session:
    name: session_with_delay
    delay: 20s

- name: Retrieve info about session by id
  community.general.consul_session:
    id: session_id
    state: info

- name: Retrieve active sessions
  community.general.consul_session:
    state: list

- name: Register session with a ttl
  community.general.consul_session:
    name: session-with-ttl
    ttl: 600 # sec
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._consul import AUTH_ARGUMENTS_SPEC, _ConsulModule


def execute(module, consul_module):
    state = module.params.get("state")

    if state in ["info", "list", "node"]:
        lookup_sessions(module, consul_module)
    elif state == "present":
        update_session(module, consul_module)
    else:
        remove_session(module, consul_module)


def list_sessions(consul_module, datacenter):
    return consul_module.get("session/list", params={"dc": datacenter})


def list_sessions_for_node(consul_module, node, datacenter):
    return consul_module.get(("session", "node", node), params={"dc": datacenter})


def get_session_info(consul_module, session_id, datacenter):
    return consul_module.get(("session", "info", session_id), params={"dc": datacenter})


def lookup_sessions(module, consul_module):
    datacenter = module.params.get("datacenter")

    state = module.params.get("state")
    try:
        if state == "list":
            sessions_list = list_sessions(consul_module, datacenter)
            # Ditch the index, this can be grabbed from the results
            if sessions_list and len(sessions_list) >= 2:
                sessions_list = sessions_list[1]
            module.exit_json(changed=True, sessions=sessions_list)
        elif state == "node":
            node = module.params.get("node")
            sessions = list_sessions_for_node(consul_module, node, datacenter)
            module.exit_json(changed=True, node=node, sessions=sessions)
        elif state == "info":
            session_id = module.params.get("id")

            session_by_id = get_session_info(consul_module, session_id, datacenter)
            module.exit_json(changed=True, session_id=session_id, sessions=session_by_id)

    except Exception as e:
        module.fail_json(msg=f"Could not retrieve session info {e}")


def create_session(consul_module, name, behavior, ttl, node, lock_delay, datacenter, checks):
    create_data = {
        "LockDelay": lock_delay,
        "Node": node,
        "Name": name,
        "Checks": checks,
        "Behavior": behavior,
    }
    if ttl is not None:
        create_data["TTL"] = f"{ttl}s"  # TTL is in seconds
    create_session_response_dict = consul_module.put("session/create", params={"dc": datacenter}, data=create_data)
    return create_session_response_dict["ID"]


def update_session(module, consul_module):
    name = module.params.get("name")
    delay = module.params.get("delay")
    checks = module.params.get("checks")
    datacenter = module.params.get("datacenter")
    node = module.params.get("node")
    behavior = module.params.get("behavior")
    ttl = module.params.get("ttl")

    try:
        session = create_session(
            consul_module,
            name=name,
            behavior=behavior,
            ttl=ttl,
            node=node,
            lock_delay=delay,
            datacenter=datacenter,
            checks=checks,
        )
        module.exit_json(
            changed=True,
            session_id=session,
            name=name,
            behavior=behavior,
            ttl=ttl,
            delay=delay,
            checks=checks,
            node=node,
        )
    except Exception as e:
        module.fail_json(msg=f"Could not create/update session {e}")


def destroy_session(consul_module, session_id):
    return consul_module.put(("session", "destroy", session_id))


def remove_session(module, consul_module):
    session_id = module.params.get("id")

    try:
        destroy_session(consul_module, session_id)

        module.exit_json(changed=True, session_id=session_id)
    except Exception as e:
        module.fail_json(msg=f"Could not remove session with id '{session_id}' {e}")


def main():
    argument_spec = dict(
        checks=dict(type="list", elements="str"),
        delay=dict(type="int", default="15"),
        behavior=dict(type="str", default="release", choices=["release", "delete"]),
        ttl=dict(type="int"),
        id=dict(type="str"),
        name=dict(type="str"),
        node=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "info", "list", "node", "present"]),
        datacenter=dict(type="str"),
        **AUTH_ARGUMENTS_SPEC,
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=[
            ("state", "node", ["name"]),
            ("state", "info", ["id"]),
            ("state", "remove", ["id"]),
        ],
        supports_check_mode=False,
    )
    consul_module = _ConsulModule(module)

    try:
        execute(module, consul_module)
    except Exception as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_appliance_radio_settings
short_description: Resource module for devices _appliance _radio _settings
description:
  - Manage operation update of the resource devices _appliance _radio _settings.
  - Update the radio settings of an appliance.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  fiveGhzSettings:
    description: Manual radio settings for 5 GHz.
    suboptions:
      channel:
        description: Sets a manual channel for 5 GHz. Can be '36', '40', '44', '48',
          '52', '56', '60', '64', '100', '104', '108', '112', '116', '120', '124',
          '128', '132', '136', '140', '144', '149', '153', '157', '161', '165', '169',
          '173' or '177' or null for using auto channel.
        type: int
      channelWidth:
        description: Sets a manual channel width for 5 GHz. Can be '0', '20', '40',
          '80' or '160' or null for using auto channel width.
        type: int
      targetPower:
        description: Set a manual target power for 5 GHz (dBm). Enter null for using
          auto power range.
        type: int
    type: dict
  rfProfileId:
    description: The ID of an RF profile to assign to the device. If the value of
      this parameter is null, the appropriate basic RF profile (indoor or outdoor)
      will be assigned to the device. Assigning an RF profile will clear ALL manually
      configured overrides on the device (channel width, channel, power).
    type: str
  serial:
    description: Serial path parameter.
    type: str
  twoFourGhzSettings:
    description: Manual radio settings for 2.4 GHz.
    suboptions:
      channel:
        description: Sets a manual channel for 2.4 GHz. Can be '1', '2', '3', '4',
          '5', '6', '7', '8', '9', '10', '11', '12', '13' or '14' or null for using
          auto channel.
        type: int
      targetPower:
        description: Set a manual target power for 2.4 GHz (dBm). Enter null for using
          auto power range.
        type: int
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateDeviceApplianceRadioSettings
    description: Complete reference of the updateDeviceApplianceRadioSettings API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device-appliance-radio-settings
notes:
  - SDK Method used are
    appliance.Appliance.update_device_appliance_radio_settings,
  - Paths used are
    put /devices/{serial}/appliance/radio/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_appliance_radio_settings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    fiveGhzSettings:
      channel: 149
      channelWidth: 20
      targetPower: 15
    rfProfileId: '1234'
    serial: string
    twoFourGhzSettings:
      channel: 11
      targetPower: 21
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "serial": "string",
      "rfProfileId": "string",
      "twoFourGhzSettings": {
        "channel": 0,
        "targetPower": 0
      },
      "fiveGhzSettings": {
        "channel": 0,
        "channelWidth": 0,
        "targetPower": 0
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2022, Ansible Project
# Copyright (c) 2022, VMware, Inc. All Rights Reserved.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: iso_customize
short_description: Add/remove/change files in ISO file
description:
  - This module is used to add/remove/change files in ISO file.
  - The file inside ISO is overwritten if it exists by option O(add_files).
author:
  - Yuhua Zou (@ZouYuhua) <zouy@vmware.com>
requirements:
  - "pycdlib"
version_added: '5.8.0'

extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  src_iso:
    description:
      - This is the path of source ISO file.
    type: path
    required: true
  dest_iso:
    description:
      - The path of the customized ISO file.
    type: path
    required: true
  delete_files:
    description:
      - Absolute paths for files inside the ISO file that should be removed.
    type: list
    elements: str
    default: []
  add_files:
    description:
      - Allows to add and replace files in the ISO file.
      - It creates intermediate folders inside the ISO file when they do not exist.
    type: list
    elements: dict
    default: []
    suboptions:
      src_file:
        description:
          - The path with file name on the machine the module is executed on.
        type: path
        required: true
      dest_file:
        description:
          - The absolute path of the file inside the ISO file.
        type: str
        required: true
notes:
  - The function C(add_file) in pycdlib is designed to overwrite the existing file in ISO with type ISO9660 / Rock Ridge 1.12
    / Joliet / UDF. But it does not overwrite the existing file in ISO with Rock Ridge 1.09 / 1.10. So we take workaround
    "delete the existing file and then add file for ISO with Rock Ridge".
"""

EXAMPLES = r"""
- name: "Customize ISO file"
  community.general.iso_customize:
    src_iso: "/path/to/ubuntu-22.04-desktop-amd64.iso"
    dest_iso: "/path/to/ubuntu-22.04-desktop-amd64-customized.iso"
    delete_files:
      - "/boot.catalog"
    add_files:
      - src_file: "/path/to/grub.cfg"
        dest_file: "/boot/grub/grub.cfg"
      - src_file: "/path/to/ubuntu.seed"
        dest_file: "/preseed/ubuntu.seed"
  register: customize_iso_result
"""

RETURN = r"""
src_iso:
  description: Path of source ISO file.
  returned: on success
  type: str
  sample: "/path/to/file.iso"
dest_iso:
  description: Path of the customized ISO file.
  returned: on success
  type: str
  sample: "/path/to/customized.iso"
"""

import os

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils import _deps as deps

with deps.declare("pycdlib"):
    import pycdlib


# The upper dir exist, we only add subdirectoy
def iso_add_dir(module, opened_iso, iso_type, dir_path):
    parent_dir, check_dirname = dir_path.rsplit("/", 1)
    if not parent_dir.strip():
        parent_dir = "/"
    check_dirname = check_dirname.strip()

    for dirname, dirlist, dummy_filelist in opened_iso.walk(iso_path=parent_dir.upper()):
        if dirname == parent_dir.upper():
            if check_dirname.upper() in dirlist:
                return

            if parent_dir == "/":
                current_dirpath = f"/{check_dirname}"
            else:
                current_dirpath = f"{parent_dir}/{check_dirname}"

            current_dirpath_upper = current_dirpath.upper()
            try:
                if iso_type == "iso9660":
                    opened_iso.add_directory(current_dirpath_upper)
                elif iso_type == "rr":
                    opened_iso.add_directory(current_dirpath_upper, rr_name=check_dirname)
                elif iso_type == "joliet":
                    opened_iso.add_directory(current_dirpath_upper, joliet_path=current_dirpath)
                elif iso_type == "udf":
                    opened_iso.add_directory(current_dirpath_upper, udf_path=current_dirpath)
            except Exception as err:
                msg = f"Failed to create dir {current_dirpath} with error: {err}"
                module.fail_json(msg=msg)


def iso_add_dirs(module, opened_iso, iso_type, dir_path):
    dirnames = dir_path.strip().split("/")

    current_dirpath = "/"
    for item in dirnames:
        if not item.strip():
            continue
        if current_dirpath == "/":
            current_dirpath = f"/{item}"
        else:
            current_dirpath = f"{current_dirpath}/{item}"

        iso_add_dir(module, opened_iso, iso_type, current_dirpath)


def iso_check_file_exists(opened_iso, dest_file):
    file_dir = os.path.dirname(dest_file).strip()
    file_name = os.path.basename(dest_file)
    dirnames = file_dir.strip().split("/")

    parent_dir = "/"
    for item in dirnames:
        if not item.strip():
            continue

        for dirname, dirlist, dummy_filelist in opened_iso.walk(iso_path=parent_dir.upper()):
            if dirname != parent_dir.upper():
                break

            if item.upper() not in dirlist:
                return False

        if parent_dir == "/":
            parent_dir = f"/{item}"
        else:
            parent_dir = f"{parent_dir}/{item}"

    if "." not in file_name:
        file_in_iso_path = f"{file_name.upper()}.;1"
    else:
        file_in_iso_path = f"{file_name.upper()};1"

    for dirname, dummy_dirlist, filelist in opened_iso.walk(iso_path=parent_dir.upper()):
        if dirname != parent_dir.upper():
            return False

        return file_name.upper() in filelist or file_in_iso_path in filelist


def iso_add_file(module, opened_iso, iso_type, src_file, dest_file):
    dest_file = dest_file.strip()
    if dest_file[0] != "/":
        dest_file = f"/{dest_file}"

    file_local = src_file.strip()

    file_dir = os.path.dirname(dest_file).strip()
    file_name = os.path.basename(dest_file)
    if "." not in file_name:
        file_in_iso_path = f"{dest_file.upper()}.;1"
    else:
        file_in_iso_path = f"{dest_file.upper()};1"

    if file_dir and file_dir != "/":
        iso_add_dirs(module, opened_iso, iso_type, file_dir)

    try:
        if iso_type == "iso9660":
            opened_iso.add_file(file_local, iso_path=file_in_iso_path)
        elif iso_type == "rr":
            # For ISO with Rock Ridge 1.09 / 1.10, it won't overwrite the existing file
            # So we take workaround here: delete the existing file and then add file
            if iso_check_file_exists(opened_iso, dest_file):
                opened_iso.rm_file(iso_path=file_in_iso_path)
            opened_iso.add_file(file_local, iso_path=file_in_iso_path, rr_name=file_name)
        elif iso_type == "joliet":
            opened_iso.add_file(file_local, iso_path=file_in_iso_path, joliet_path=dest_file)
        elif iso_type == "udf":
            # For ISO with UDF, it won't always succeed to overwrite the existing file
            # So we take workaround here: delete the existing file and then add file
            if iso_check_file_exists(opened_iso, dest_file):
                opened_iso.rm_file(udf_path=dest_file)
            opened_iso.add_file(file_local, iso_path=file_in_iso_path, udf_path=dest_file)
    except Exception as err:
        msg = f"Failed to add local file {file_local} to ISO with error: {err}"
        module.fail_json(msg=msg)


def iso_delete_file(module, opened_iso, iso_type, dest_file):
    dest_file = dest_file.strip()
    if dest_file[0] != "/":
        dest_file = f"/{dest_file}"
    file_name = os.path.basename(dest_file)

    if not iso_check_file_exists(opened_iso, dest_file):
        module.fail_json(msg=f"The file {dest_file} does not exist.")

    if "." not in file_name:
        file_in_iso_path = f"{dest_file.upper()}.;1"
    else:
        file_in_iso_path = f"{dest_file.upper()};1"

    try:
        if iso_type == "iso9660":
            opened_iso.rm_file(iso_path=file_in_iso_path)
        elif iso_type == "rr":
            opened_iso.rm_file(iso_path=file_in_iso_path)
        elif iso_type == "joliet":
            opened_iso.rm_file(joliet_path=dest_file)
        elif iso_type == "udf":
            opened_iso.rm_file(udf_path=dest_file)
    except Exception as err:
        msg = f"Failed to delete iso file {dest_file} with error: {err}"
        module.fail_json(msg=msg)


def iso_rebuild(module, src_iso, dest_iso, delete_files_list, add_files_list):
    iso = None
    iso_type = "iso9660"

    try:
        iso = pycdlib.PyCdlib(always_consistent=True)
        iso.open(src_iso)
        if iso.has_rock_ridge():
            iso_type = "rr"
        elif iso.has_joliet():
            iso_type = "joliet"
        elif iso.has_udf():
            iso_type = "udf"

        for item in delete_files_list:
            iso_delete_file(module, iso, iso_type, item)

        for item in add_files_list:
            iso_add_file(module, iso, iso_type, item["src_file"], item["dest_file"])

        iso.write(dest_iso)
    except Exception as err:
        msg = f"Failed to rebuild ISO {src_iso} with error: {err}"
        module.fail_json(msg=msg)
    finally:
        if iso:
            iso.close()


def main():
    argument_spec = dict(
        src_iso=dict(type="path", required=True),
        dest_iso=dict(type="path", required=True),
        delete_files=dict(type="list", elements="str", default=[]),
        add_files=dict(
            type="list",
            elements="dict",
            default=[],
            options=dict(
                src_file=dict(type="path", required=True),
                dest_file=dict(type="str", required=True),
            ),
        ),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=[
            ("delete_files", "add_files"),
        ],
        supports_check_mode=True,
    )
    deps.validate(module)

    src_iso = module.params["src_iso"]
    if not os.path.exists(src_iso):
        module.fail_json(msg=f"ISO file {src_iso} does not exist.")

    dest_iso = module.params["dest_iso"]
    dest_iso_dir = os.path.dirname(dest_iso)
    if dest_iso_dir and not os.path.exists(dest_iso_dir):
        module.fail_json(msg=f"The dest directory {dest_iso_dir} does not exist")

    delete_files_list = [s.strip() for s in module.params["delete_files"]]
    add_files_list = module.params["add_files"]
    if add_files_list:
        for item in add_files_list:
            if not os.path.exists(item["src_file"]):
                module.fail_json(msg=f"The file {item['src_file']} does not exist.")

    result = dict(
        src_iso=src_iso,
        customized_iso=dest_iso,
        delete_files=delete_files_list,
        add_files=add_files_list,
        changed=True,
    )

    if not module.check_mode:
        iso_rebuild(module, src_iso, dest_iso, delete_files_list, add_files_list)

    result["changed"] = True
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ip_pool
short_description: IP Pool configuration for Cisco Intersight
description:
  - IP Pool configuration for Cisco Intersight.
  - Used to configure IP pools settings on Cisco Intersight managed devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the IP Pool.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the IP Pool.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  assignment_order:
    description:
      - Assignment order decides the order in which the next identifier is allocated.
      - If C(sequential), identifiers are assigned in a sequential order.
      - If C(default), the system determines the assignment order.
    type: str
    choices: [default, sequential]
    default: default
  enable_block_level_subnet_config:
    description:
      - Determines if the "Netmask", "Gateway", "PrimaryDns" and "SecondaryDns" is globally defined or specified per IPv4 block.
      - Determines if the "Prefix", "Gateway", "PrimaryDns" and "SecondaryDns" is globally defined or specified per IPv6 block.
    type: bool
    default: false
  ipv4_config:
    description:
      - Global definition of IPv4 "Netmask", "Gateway", "PrimaryDns" and "SecondaryDns".
      - Used only when "enable_block_level_subnet_config" is false, otherwise should be defined inside ipv4_blocks.
    type: dict
    suboptions:
      netmask:
        description:
          - A subnet mask is a 32-bit number that masks an IP address and divides the IP address into network address and host address.
          - Netmask used for all IPv4 blocks.
        type: str
      gateway:
        description:
          - IP address of the default IPv4 gateway.
          - Gateway IP used for all IPv4 blocks.
        type: str
      primary_dns:
        description:
          - IP Address of the primary Domain Name System (DNS) server.
          - PrimaryDns IP used for all IPv4 blocks.
        type: str
      secondary_dns:
        description:
          - IP Address of the secondary Domain Name System (DNS) server.
          - SecondaryDns IP used for all IPv4 blocks.
        type: str
  ipv4_blocks:
    description:
      - List of the IPv4 blocks.
      - Should include the ipv4_config details per block in case "enable_block_level_subnet_config" is true
    type: list
    elements: dict
    suboptions:
      from:
        description:
          - First IPv4 address of the block.
        type: str
      size:
        description:
          - Number of identifiers this block can hold.
        type: int
      ipv4_config:
        description:
          - Block definition of IPv4 "Netmask", "Gateway", "PrimaryDns" and "SecondaryDns".
          - Used only when "enable_block_level_subnet_config" is true, otherwise should be defined globally.
        type: dict
        suboptions:
          netmask:
            description:
              - The Netmask used for the current IPv4 block.
            type: str
          gateway:
            description:
              - The Gateway IP used for the current IPv4 block.
            type: str
          primary_dns:
            description:
              - The PrimaryDns IP used for the current IPv4 block.
            type: str
          secondary_dns:
            description:
              - The SecondaryDns IP used for the current IPv4 block.
            type: str
  ipv6_config:
    description:
      - Global definition of IPv6 "Prefix", "Gateway", "PrimaryDns" and "SecondaryDns".
      - Used only when "enable_block_level_subnet_config" is false, otherwise should be defined inside ipv6_blocks.
    type: dict
    suboptions:
      prefix:
        description:
          - A prefix length which masks the IP address and divides the IP address into network address and host address.
          - Prefix used for all IPv6 blocks.
        type: str
      gateway:
        description:
          - IP address of the default IPv6 gateway.
          - Gateway IP used for all IPv6 blocks.
        type: str
      primary_dns:
        description:
          - IP Address of the primary Domain Name System (DNS) server.
          - PrimaryDns IP used for all IPv6 blocks.
        type: str
      secondary_dns:
        description:
          - IP Address of the secondary Domain Name System (DNS) server.
          - SecondaryDns IP used for all IPv6 blocks.
        type: str
  ipv6_blocks:
    description:
      - Define the IPv6 blocks.
      - Should include the ipv6_config details per block in case "enable_block_level_subnet_config" is true
    type: list
    elements: dict
    suboptions:
      from:
        description:
          - The initial IP address for the IPv6 block.
        type: str
      size:
        description:
          - The number of IPs in the block.
        type: int
      ipv6_config:
        description:
          - Block definition of IPv6 "Prefix", "Gateway", "PrimaryDns" and "SecondaryDns".
          - Used only when "enable_block_level_subnet_config" is true, otherwise should be defined globally.
        type: dict
        suboptions:
          prefix:
            description:
              - The Prefix used for the current IPv6 block.
            type: int
          gateway:
            description:
              - The Gateway IP used for the current IPv6 block.
            type: str
          primary_dns:
            description:
              - The PrimaryDns IP used for the current IPv6 block.
            type: str
          secondary_dns:
            description:
              - The SecondaryDns IP used for the current IPv6 block.
            type: str
author:
  - Shahar Golshani (@sgolshan)
'''

EXAMPLES = r'''
- name: Configure IP Pool with global config
  cisco.intersight.intersight_ip_pool:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-ip-pool
    description: IP Pool for lab use
    assignment_order: sequential
    ipv4_config:
      netmask: "255.255.255.0"
      gateway: "172.17.116.1"
      primary_dns: "172.17.116.2"
      secondary_dns: "172.17.116.3"
    ipv4_blocks:
      - from: "172.17.116.32"
        size: 16
      - from: "172.17.116.64"
        size: 16
    ipv6_config:
      prefix: 64
      gateway: "2001:db8::1"
      primary_dns: "2001:4860:4860::8888"
      secondary_dns: "2001:4860:4860::8844"
    ipv6_blocks:
      - from: "2001:db8::2"
        size: 64
      - from: "2001:db8::42"
        size: 64
    tags:
      - Key: Site
        Value: RCDN


- name: Configure IP Pool with block level config
  cisco.intersight.intersight_ip_pool:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-ip-pool
    description: IP Pool for lab use
    enable_block_level_subnet_config: true
    ipv4_blocks:
      - from: "172.17.116.32"
        size: 16
        ipv4_config:
          netmask: "255.255.255.0"
          gateway: "172.17.116.1"
          primary_dns: "172.17.116.2"
          secondary_dns: "172.17.116.3"
      - from: "172.17.117.32"
        size: 16
        ipv4_config:
          netmask: "255.255.255.0"
          gateway: "172.17.117.1"
          primary_dns: "172.17.117.2"
          secondary_dns: "172.17.117.3"
    ipv6_blocks:
      - from: "2001:db8::2"
        size: 64
        ipv6_config:
          prefix: 64
          gateway: "2001:db8::1"
          primary_dns: "2001:4860:4860::8888"
          secondary_dns: "2001:4860:4860::8844"
      - from: "fd12:3456:789a::1"
        size: 64
        ipv6_config:
          prefix: 64
          gateway: "fd12:3456:789a::1"
          primary_dns: "2606:4700:4700::1111"
          secondary_dns: "2606:4700:4700::1001"
    tags:
      - Key: Site
        Value: RCDN


- name: Delete IP Pool
  cisco.intersight.intersight_ip_pool:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-ip-pool
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "AccountMoid": "",
        "Ancestors": [],
        "Assigned": 0,
        "AssignmentOrder": "sequential",
        "ClassId": "ippool.Pool",
        "CreateTime": "",
        "Description": "IP Pool for lab use",
        "DomainGroupMoid": "",
        "EnableBlockLevelSubnetConfig": true,
        "IpV4Blocks": [],
        "IpV4Config": {},
        "IpV6Blocks": [],
        "IpV6Config": {},
        "ModTime": "",
        "Moid": "",
        "Name": "lab-ip-pool",
        "ObjectType": "ippool.Pool",
        "Organization": {},
        "Owners": [],
        "PermissionResources": [],
        "Reservations": [],
        "Reserved": 0,
        "ShadowPools": [],
        "SharedScope": "",
        "Size": 160,
        "Tags": [
            {
                "Key": "Site",
                "Value": "RCDN"
            }
        ],
        "V4Assigned": 0,
        "V4Size": 32,
        "V6Assigned": 0,
        "V6Size": 128
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        assignment_order=dict(type='str', choices=['default', 'sequential'], default='default'),
        enable_block_level_subnet_config=dict(type='bool', default=False),
        ipv4_config=dict(type='dict'),
        ipv4_blocks=dict(type='list', elements='dict'),
        ipv6_config=dict(type='dict'),
        ipv6_blocks=dict(type='list', elements='dict'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/ippool/Pools'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'EnableBlockLevelSubnetConfig': intersight.module.params['enable_block_level_subnet_config'],
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['AssignmentOrder'] = intersight.module.params['assignment_order']

        # Validate that at least one of ipv4_blocks/ipv6_blocks was passed. We don't mark it as required in order to support absent.
        if not intersight.module.params['ipv4_blocks'] and not intersight.module.params['ipv6_blocks']:
            module.fail_json(msg="at least one of ipv4_blocks/ipv6_blocks parameters must be provided and contain at least one block when state is present")
        # Validate that when enable_block_level_subnet_config is true, ipv4_blocks/ipv6_blocks contains ipv4_config/ipv6_config.
        if intersight.module.params['enable_block_level_subnet_config']:
            if intersight.module.params['ipv4_blocks']:
                for block in intersight.module.params['ipv4_blocks']:
                    if "ipv4_config" not in block:
                        module.fail_json(msg="a block in ipv4_blocks is missing ipv4_config")
            if intersight.module.params['ipv6_blocks']:
                for block in intersight.module.params['ipv6_blocks']:
                    if "ipv6_config" not in block:
                        module.fail_json(msg="a block in ipv6_blocks is missing ipv6_config")
        # Validate that when enable_block_level_subnet_config is false, ipv4_blocks/ipv6_blocks has a global ipv4_config/ipv6_config.
        else:
            if (intersight.module.params['ipv4_blocks'] is None) != (intersight.module.params['ipv4_config'] is None):
                module.fail_json(msg="when enable_block_level_subnet_config is false, ipv4_blocks should be configured with global ipv4_config")
            if (intersight.module.params['ipv6_blocks'] is None) != (intersight.module.params['ipv6_config'] is None):
                module.fail_json(msg="when enable_block_level_subnet_config is false, ipv6_blocks should be configured with global ipv6_config")

        IpV4Blocks = []
        if intersight.module.params['ipv4_blocks']:
            for v4_block in intersight.module.params['ipv4_blocks']:
                block = {}
                block['From'] = v4_block['from']
                block['Size'] = v4_block['size']
                if 'ipv4_config' in v4_block:
                    block['IpV4Config'] = {
                        'Netmask': v4_block['ipv4_config']['netmask'],
                        'Gateway': v4_block['ipv4_config']['gateway'],
                        'PrimaryDns': v4_block['ipv4_config']['primary_dns'],
                        'SecondaryDns': v4_block['ipv4_config']['secondary_dns']
                    }
                IpV4Blocks.append(block)
        intersight.api_body['IpV4Blocks'] = IpV4Blocks

        IpV6Blocks = []
        if intersight.module.params['ipv6_blocks']:
            for v6_block in intersight.module.params['ipv6_blocks'] :
                block = {}
                block['From'] = v6_block['from']
                block['Size'] = v6_block['size']
                if 'ipv6_config' in v6_block:
                    block['IpV6Config'] = {
                        'Prefix': v6_block['ipv6_config']['prefix'],
                        'Gateway': v6_block['ipv6_config']['gateway'],
                        'PrimaryDns': v6_block['ipv6_config']['primary_dns'],
                        'SecondaryDns': v6_block['ipv6_config']['secondary_dns']
                    }
                IpV6Blocks.append(block)
        intersight.api_body['IpV6Blocks'] = IpV6Blocks

        if not intersight.module.params['enable_block_level_subnet_config']:
            if intersight.module.params['ipv4_config']:
                intersight.api_body['IpV4Config'] = {
                    'Netmask': intersight.module.params['ipv4_config']['netmask'],
                    'Gateway': intersight.module.params['ipv4_config']['gateway'],
                    'PrimaryDns': intersight.module.params['ipv4_config']['primary_dns'],
                    'SecondaryDns': intersight.module.params['ipv4_config']['secondary_dns'],
                }
            else:
                intersight.api_body['IpV4Config'] = None

            if intersight.module.params['ipv6_config']:
                intersight.api_body['IpV6Config'] = {
                    'Prefix': intersight.module.params['ipv6_config']['prefix'],
                    'Gateway': intersight.module.params['ipv6_config']['gateway'],
                    'PrimaryDns': intersight.module.params['ipv6_config']['primary_dns'],
                    'SecondaryDns': intersight.module.params['ipv6_config']['secondary_dns'],
                }
            else:
                intersight.api_body['IpV6Config'] = None

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The comp_type5 filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: comp_type5
author: Ken Celenza (@itdependsnetworks)
version_added: "1.0.0"
short_description: The comp_type5 filter plugin.
description:
  - The filter confirms configuration idempotency on use of type5_pw.
notes:
  - The filter confirms configuration idempotency on use of type5_pw.
  - Can be used to validate password post hashing
    username cisco secret 5 {{ ansible_ssh_pass | ansible.netcommon.comp_type5(encrypted, True) }}
options:
  unencrypted_password:
    description:
    - The unencrypted text.
    type: str
    required: True
  encrypted_password:
    description:
    - The encrypted text.
    type: str
    required: True
  return_original:
    description:
    - Return the original text.
    type: bool
"""

EXAMPLES = r"""
# Using comp_type5

# playbook

- name: Set the facts
  ansible.builtin.set_fact:
    unencrypted_password: "cisco@123"
    encrypted_password: "$1$avs$uSTOEMh65ADDBREAKqzvpb9yBMpzd/"

- name: Invoke comp_type5
  ansible.builtin.debug:
    msg: "{{ unencrypted_password | ansible.netcommon.comp_type5(encrypted_password, False) }}"

# Task Output
# -----------
#
# TASK [Set the facts]
# ok: [35.155.113.92] => changed=false
#   ansible_facts:
#     encrypted_password: $1$avs$uSTOEMh65ADDBREAKqzvpb9yBMpzd/
#     unencrypted_password: cisco@123

# TASK [Invoke comp_type5]
# ok: [35.155.113.92] =>
#   msg: true
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.argspec_filter_utils import (
    convert_to_native,
)
from ansible_collections.ansible.netcommon.plugins.plugin_utils.comp_type5 import comp_type5


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _comp_type5(*args, **kwargs):
    """Extend vlan data"""

    keys = [
        "unencrypted_password",
        "encrypted_password",
        "return_original",
    ]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    data = convert_to_native(data)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="comp_type5")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return comp_type5(**updated_data)


class FilterModule(object):
    """comp_type5"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"comp_type5": _comp_type5}



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdoriginsoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdorigin_info
version_added: "3.4.0"
short_description: Get Azure Front Door Origin facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) Origin or all AFD Origins.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    resource_group:
        description:
            - Name of the resource group where this AFD Profile belongs.
        required: true
        type: str
    origin_group_name:
        description:
            - Name of the origin group which is unique within the profile.
        required: true
        type: str
    profile_name:
        description:
            - Name of the AFD profile.
        required: true
        type: str
    name:
        description:
            - Limit results to a specific AFD Origin.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all Origins in AFD Profile
  azure_rm_afdorigin_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile

- name: Get facts of specific AFD Origin
  azure_rm_afdorigin_info:
    name: myOrigin1
    profile_name: myCDNProfile
    resource_group: myResourceGroup
'''

RETURN = '''
afdorigins:
    description: List of AFD Origins.
    returned: always
    type: complex
    contains:
        azure_origin:
            description:
                - Resource reference to the AFD origin resource.
            type: str
        deployment_status:
            description:
                - Current state of the resource.
            type: str
            sample: NotStarted
        enabled_state:
            description:
                - Whether to enable health probes to be made against backends defined under backend pools.
                - Health probes can only be disabled if there is a single enabled backend in single enabled backend pool.
            type: str
            sample: Enabled
        host_name:
            description:
                - The address of the origin. Domain names, IPv4 addresses, and IPv6 addresses are supported.
                - This should be unique across all origins in an endpoint.
            type: str
        http_port:
            description:
                - The value of the HTTP port. Must be between 1 and 65535.
            type: int
            sample: 80
        https_port:
            description:
                - The value of the HTTPS port. Must be between 1 and 65535.
            type: int
            sample: 443
        id:
            description:
                - ID of the AFD Origin.
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/origingroups/myOG/origins/myO"
        name:
            description:
                - Name of the AFD Origin.
            type: str
        origin_group_name:
            description:
                - Name of the origin group which is unique within the profile.
            type: str
        origin_host_header:
            description:
                - The host header value sent to the origin with each request. If you leave this blank, the request hostname determines this value.
                - Azure Front Door origins, such as Web Apps, Blob Storage, and Cloud Services require this host header value to match the origin
                - hostname by default. This overrides the host header defined at the AFD Endpoint.
            type: str
        priority:
            description:
                - Priority of origin in given origin group for load balancing. Higher priorities will not be used for load balancing if any
                - lower priority origin is healthy. Must be between 1 and 5.
            type: int
        profile_name:
            description:
                - Name of the AFD Profile where the Origin will be added.
            type: str
        provisioning_state:
            description:
                - Provisioning status of the AFD Origin.
            type: str
            sample: Succeeded
        resource_group:
            description:
                - Name of a resource group where the AFD Origin exists or will be created.
            type: str
        shared_private_link_resource:
            description:
                - The properties of the private link resource for private origin.
            type: dict
            contains:
                group_id:
                    description:
                        - The group id from the provider of resource the shared private link resource is for.
                    type: str
                private_link:
                    description:
                        - The resource id of the resource the shared private link resource is for.
                    type: str
                private_link_location:
                    description:
                        - The location of the shared private link resource.
                    type: str
                request_message:
                    description:
                        - The request message for requesting approval of the shared private link resource.
                    type: str
                status:
                    description:
                        - Status of the shared private link resource. Can be Pending, Approved, Rejected, Disconnected, or Timeout.
                    type: str
                    sample: Approved
        type:
            description:
                - Resource type.
            type: str
            sample: Microsoft.Cdn/profiles/origingroups/origins
        weight:
            description:
                - Weight of the origin in given origin group for load balancing. Must be between 1 and 1000.
            type: int
'''

import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'AFDOrigin'


class AzureRMAFDOriginInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD Origin facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            origin_group_name=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            )
        )

        self.results = dict(
            changed=False,
            afdorigins=[]
        )

        self.name = None
        self.origin_group_name = None
        self.resource_group = None
        self.profile_name = None

        super(AzureRMAFDOriginInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdorigins'] = self.get_item()
        else:
            self.results['afdorigins'] = self.list_by_profile()

        return self.results

    def get_item(self):
        """Get a single Azure AFD Origin"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.afd_origins.get(resource_group_name=self.resource_group,
                                                   profile_name=self.profile_name,
                                                   origin_group_name=self.origin_group_name,
                                                   origin_name=self.name)
        except Exception as exc:
            self.log("Did not find resource. {0}".format(str(exc)))

        if item:
            result = [self.serialize_afdorigin(item)]

        return result

    def list_by_profile(self):
        """Get all Azure AFD Origins within an AFD profile"""

        self.log('List all AFD Origins within an AFD profile')

        try:
            response = self.cdn_client.afd_origins.list_by_origin_group(resource_group_name=self.resource_group,
                                                                        profile_name=self.profile_name,
                                                                        origin_group_name=self.origin_group_name)
        except Exception as exc:
            self.fail('Failed to list all origins. {0}'.format(str(exc)))

        results = []
        for item in response:
            results.append(self.serialize_afdorigin(item))

        return results

    def serialize_afdorigin(self, afdorigin):
        '''
        Convert a AFD Origin object to dict.
        :param afdorigin: AFD Origin object
        :return: dict
        '''
        result = self.serialize_obj(afdorigin, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['azure_origin'] = afdorigin.azure_origin
        new_result['deployment_status'] = afdorigin.deployment_status
        new_result['enabled_state'] = afdorigin.enabled_state
        new_result['host_name'] = afdorigin.host_name
        new_result['http_port'] = afdorigin.http_port
        new_result['https_port'] = afdorigin.https_port
        new_result['https_port'] = afdorigin.https_port
        new_result['id'] = afdorigin.id
        new_result['name'] = afdorigin.name
        new_result['origin_group_name'] = re.sub('\\/.*', '', re.sub('.*origingroups\\/', '', result['id']))
        new_result['origin_host_header'] = afdorigin.origin_host_header
        new_result['priority'] = afdorigin.priority
        new_result['provisioning_state'] = afdorigin.provisioning_state
        new_result['priority'] = afdorigin.priority
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['resource_group'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['shared_private_link_resource'] = dict()
        if afdorigin.shared_private_link_resource:
            new_result['shared_private_link_resource']['group_id'] = afdorigin.shared_private_link_resource.group_id
            new_result['shared_private_link_resource']['private_link'] = afdorigin.shared_private_link_resource.private_link
            new_result['shared_private_link_resource']['private_link_location'] = afdorigin.shared_private_link_resource.private_link_location
            new_result['shared_private_link_resource']['request_message'] = afdorigin.shared_private_link_resource.request_message
            new_result['shared_private_link_resource']['status'] = afdorigin.shared_private_link_resource.status
        else:
            new_result['shared_private_link_resource']['group_id'] = None
            new_result['shared_private_link_resource']['private_link'] = None
            new_result['shared_private_link_resource']['private_link_location'] = None
            new_result['shared_private_link_resource']['request_message'] = None
            new_result['shared_private_link_resource']['status'] = None
        new_result['type'] = afdorigin.type
        new_result['weight'] = afdorigin.weight
        return new_result


def main():
    """Main module execution code path"""
    AzureRMAFDOriginInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Faiz Mohammad (@Ziaf007) <faizmoh@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_node_mgmt_epg_to_contract
version_added: "2.12.0"
short_description: Bind Node Management EPGs to Contracts (fv:RsCons, fv:RsProv, fv:RsProtBy, fv:RsConsIf and mgmt:RsOoBProv)
description:
- Bind Node Management EPGs to Contracts on Cisco ACI fabrics.
notes:
- The O(epg) and O(contract) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_node_mgmt_epg), M(cisco.aci.aci_oob_contract) and M(cisco.aci.aci_contract) modules can be used for this.
options:
  contract:
    description:
    - The name of the contract or contract interface.
    type: str
    aliases: [ contract_name, contract_interface ]
  contract_type:
    description:
    - Determines the type of the Contract.
    type: str
    required: true
    choices: [ consumer, provider, taboo, interface ]
  epg:
    description:
    - The name of the Node Management end point group.
    type: str
    aliases: [ epg_name ]
  epg_type:
    description:
    - The type of the Node Management end point group.
    type: str
    required: true
    aliases: [ type ]
    choices: [ in_band, out_of_band ]
  priority:
    description:
    - Quality of Service (QoS) class.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
  provider_match:
    description:
    - The matching algorithm for Provided Contracts.
    - The APIC defaults to C(at_least_one) when unset during creation.
    type: str
    choices: [ all, at_least_one, at_most_one, none ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

seealso:
- module: cisco.aci.aci_node_mgmt_epg
- module: cisco.aci.aci_contract
- module: cisco.aci.aci_oob_contract
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(fv:RsCons), B(fv:RsProv), B(fv:RsProtBy), B(fv:RsConsIf), and B(mgmt:RsOoBProv).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Faiz Mohammad (@Ziaf007)
"""

EXAMPLES = r"""
- name: Add a new contract to Inband EPG binding
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    epg: anstest
    epg_type: in_band
    contract: anstest_http
    contract_type: provider
    priority: level2
    provider_match: at_least_one
    state: present
  delegate_to: localhost

- name: Add a new contract to Out-of-Band EPG binding
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    epg: anstest
    epg_type: out_of_band
    contract: anstest_http
    contract_type: provider
    priority: level3
    state: present
  delegate_to: localhost

- name: Update a contract of Inband EPG binding
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    epg: anstest
    epg_type: in_band
    contract: anstest_http
    contract_type: provider
    priority: level5
    provider_match: all
    state: present
  delegate_to: localhost

- name: Query a specific contract to EPG binding
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    epg: anstest
    epg_type: in_band
    contract: anstest_http
    contract_type: consumer
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all provider contract to EPG bindings
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    contract_type: provider
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove an existing contract to Inband EPG binding
  cisco.aci.aci_node_mgmt_epg_to_contract:
    host: apic
    username: admin
    password: SomeSecretPassword
    epg: anstest
    epg_type: in_band
    contract: anstest_http
    contract_type: consumer
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import ACI_CLASS_MAPPING, MANAGEMENT_EPG_CLASS_MAPPING, PROVIDER_MATCH_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        contract_type=dict(type="str", choices=["consumer", "provider", "taboo", "interface"], required=True),
        epg_type=dict(
            type="str", aliases=["type"], choices=["in_band", "out_of_band"], required=True
        ),  # required for querying as provider class for INB and OOB are different
        epg=dict(type="str", aliases=["epg_name"]),  # Not required for querying all objects
        contract=dict(type="str", aliases=["contract_name", "contract_interface"]),  # Not required for querying all objects
        priority=dict(type="str", choices=["level1", "level2", "level3", "level4", "level5", "level6", "unspecified"]),
        provider_match=dict(type="str", choices=["all", "at_least_one", "at_most_one", "none"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["epg", "contract"]],
            ["state", "present", ["epg", "contract"]],
        ],
    )

    epg_type = module.params.get("epg_type")
    contract = module.params.get("contract")
    contract_type = module.params.get("contract_type")
    epg = module.params.get("epg")
    priority = module.params.get("priority")
    provider_match = module.params.get("provider_match")
    if provider_match is not None:
        provider_match = PROVIDER_MATCH_MAPPING[provider_match]
    state = module.params.get("state")

    identifier = None
    if epg_type == "in_band":

        if contract_type != "provider" and provider_match is not None:
            module.fail_json(msg="the provider_match is only configurable for Provider Contracts")

        identifier = contract_type
        class_config = {"matchT": provider_match, "prio": priority, ACI_CLASS_MAPPING[contract_type]["name"]: contract}

    elif epg_type == "out_of_band":

        if contract_type != "provider":
            module.fail_json(msg="only provider contract_type is supported for out_of_band epg_type.")

        if provider_match is not None:
            module.fail_json(msg="The provider_match argument is not supported for out_of_band Provider contracts")

        identifier = "oob_provider"
        class_config = {"prio": priority, "tnVzOOBBrCPName": contract}

    aci_class = ACI_CLASS_MAPPING[identifier]["class"]
    aci_rn = ACI_CLASS_MAPPING[identifier]["rn"]
    aci_name = ACI_CLASS_MAPPING[identifier]["name"]

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="mgmtp",
            aci_rn="tn-mgmt/mgmtp-default",
            module_object=None,
        ),
        subclass_1=dict(
            aci_class=MANAGEMENT_EPG_CLASS_MAPPING[epg_type]["epg_class"],
            aci_rn="{0}{1}".format(MANAGEMENT_EPG_CLASS_MAPPING[epg_type]["epg_rn"], epg),
            module_object=epg,
            target_filter={"name": epg},
        ),
        subclass_2=dict(
            aci_class=aci_class,
            aci_rn="{0}{1}".format(aci_rn, contract),
            module_object=contract,
            target_filter={aci_name: contract},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=aci_class,
            class_config=class_config,
        )

        aci.get_diff(aci_class=aci_class)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2021, Andrew Pantuso (@ajpantuso) <ajpantuso@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: unicode_normalize
short_description: Normalizes unicode strings to facilitate comparison of characters with normalized forms
version_added: 3.7.0
author: Andrew Pantuso (@Ajpantuso)
description:
  - Normalizes unicode strings to facilitate comparison of characters with normalized forms.
positional: form
options:
  _input:
    description: A unicode string.
    type: string
    required: true
  form:
    description:
      - The normal form to use.
      - See U(https://docs.python.org/3/library/unicodedata.html#unicodedata.normalize) for details.
    type: string
    default: NFC
    choices:
      - NFC
      - NFD
      - NFKC
      - NFKD
"""

EXAMPLES = r"""
- name: Normalize unicode string
  ansible.builtin.set_fact:
    dictionary: "{{ 'ä' | community.general.unicode_normalize('NFKD') }}"
    # The resulting string has length 2: one letter is 'a', the other
    # the diacritic combiner.
"""

RETURN = r"""
_value:
  description: The normalized unicode string of the specified normal form.
  type: string
"""

from unicodedata import normalize

from ansible.errors import AnsibleFilterError

try:
    from ansible.errors import AnsibleTypeError
except ImportError:
    from ansible.errors import AnsibleFilterTypeError as AnsibleTypeError  # type: ignore


def unicode_normalize(data, form="NFC"):
    """Applies normalization to 'unicode' strings.

    Args:
        data: A unicode string piped into the Jinja filter
        form: One of ('NFC', 'NFD', 'NFKC', 'NFKD').
              See https://docs.python.org/3/library/unicodedata.html#unicodedata.normalize for more information.

    Returns:
        A normalized unicode string of the specified 'form'.
    """

    if not isinstance(data, str):
        raise AnsibleTypeError(f"{type(data)} is not a valid input type")

    if form not in ("NFC", "NFD", "NFKC", "NFKD"):
        raise AnsibleFilterError(f"{form!r} is not a valid form")

    return normalize(form, data)


class FilterModule:
    def filters(self):
        return {
            "unicode_normalize": unicode_normalize,
        }



#!/usr/bin/python
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_hostgroup
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA host-group
description:
  - Add, modify and delete an IPA host-group using IPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  append:
    description:
      - If V(true), add the listed O(host) to the O(hostgroup).
      - If V(false), only the listed O(host) is set in O(hostgroup), removing any other hosts.
    default: false
    type: bool
    version_added: 6.6.0
  cn:
    description:
      - Name of host-group.
      - Can not be changed as it is the unique identifier.
    required: true
    aliases: ["name"]
    type: str
  description:
    description:
      - Description.
    type: str
  host:
    description:
      - List of hosts that belong to the host-group.
      - If an empty list is passed all hosts are removed from the group.
      - If option is omitted hosts are not checked nor changed.
      - If option is passed all assigned hosts that are not passed are unassigned from the group.
    type: list
    elements: str
  hostgroup:
    description:
      - List of host-groups than belong to that host-group.
      - If an empty list is passed all host-groups are removed from the group.
      - If option is omitted host-groups are not checked nor changed.
      - If option is passed all assigned hostgroups that are not passed are unassigned from the group.
    type: list
    elements: str
  state:
    description:
      - State to ensure.
      - V("absent") and V("disabled") give the same results.
      - V("present") and V("enabled") give the same results.
    default: "present"
    choices: ["absent", "disabled", "enabled", "present"]
    type: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure host-group databases is present
  community.general.ipa_hostgroup:
    name: databases
    state: present
    host:
      - db.example.com
    hostgroup:
      - mysql-server
      - oracle-server
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure host-group databases is absent
  community.general.ipa_hostgroup:
    name: databases
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
"""

RETURN = r"""
hostgroup:
  description: Hostgroup as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class HostGroupIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def hostgroup_find(self, name):
        return self._post_json(method="hostgroup_find", name=None, item={"all": True, "cn": name})

    def hostgroup_add(self, name, item):
        return self._post_json(method="hostgroup_add", name=name, item=item)

    def hostgroup_mod(self, name, item):
        return self._post_json(method="hostgroup_mod", name=name, item=item)

    def hostgroup_del(self, name):
        return self._post_json(method="hostgroup_del", name=name)

    def hostgroup_add_member(self, name, item):
        return self._post_json(method="hostgroup_add_member", name=name, item=item)

    def hostgroup_add_host(self, name, item):
        return self.hostgroup_add_member(name=name, item={"host": item})

    def hostgroup_add_hostgroup(self, name, item):
        return self.hostgroup_add_member(name=name, item={"hostgroup": item})

    def hostgroup_remove_member(self, name, item):
        return self._post_json(method="hostgroup_remove_member", name=name, item=item)

    def hostgroup_remove_host(self, name, item):
        return self.hostgroup_remove_member(name=name, item={"host": item})

    def hostgroup_remove_hostgroup(self, name, item):
        return self.hostgroup_remove_member(name=name, item={"hostgroup": item})


def get_hostgroup_dict(description=None):
    data = {}
    if description is not None:
        data["description"] = description
    return data


def get_hostgroup_diff(client, ipa_hostgroup, module_hostgroup):
    return client.get_diff(ipa_data=ipa_hostgroup, module_data=module_hostgroup)


def ensure(module, client):
    name = module.params["cn"]
    state = module.params["state"]
    host = module.params["host"]
    hostgroup = module.params["hostgroup"]
    append = module.params["append"]

    ipa_hostgroup = client.hostgroup_find(name=name)
    module_hostgroup = get_hostgroup_dict(description=module.params["description"])

    changed = False
    if state in ["present", "enabled"]:
        if not ipa_hostgroup:
            changed = True
            if not module.check_mode:
                ipa_hostgroup = client.hostgroup_add(name=name, item=module_hostgroup)
        else:
            diff = get_hostgroup_diff(client, ipa_hostgroup, module_hostgroup)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_hostgroup.get(key)
                    client.hostgroup_mod(name=name, item=data)

        if host is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hostgroup.get("member_host", []),
                    [item.lower() for item in host],
                    client.hostgroup_add_host,
                    client.hostgroup_remove_host,
                    append=append,
                )
                or changed
            )

        if hostgroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hostgroup.get("member_hostgroup", []),
                    [item.lower() for item in hostgroup],
                    client.hostgroup_add_hostgroup,
                    client.hostgroup_remove_hostgroup,
                    append=append,
                )
                or changed
            )

    else:
        if ipa_hostgroup:
            changed = True
            if not module.check_mode:
                client.hostgroup_del(name=name)

    return changed, client.hostgroup_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        cn=dict(type="str", required=True, aliases=["name"]),
        description=dict(type="str"),
        host=dict(type="list", elements="str"),
        hostgroup=dict(type="list", elements="str"),
        state=dict(type="str", default="present", choices=["present", "absent", "enabled", "disabled"]),
        append=dict(type="bool", default=False),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    client = HostGroupIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, hostgroup = ensure(module, client)
        module.exit_json(changed=changed, hostgroup=hostgroup)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



# Copyright (c) 2025 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
    name: aci_inventory_system
    short_description: Cisco ACI inventory plugin
    extends_documentation_fragment:
      - cisco.aci.aci
      - constructed
    description:
        - Query details from APIC
        - Gets details on all spines and leafs behind the controller.
        - Requires a YAML configuration file whose name ends with 'cisco_aci.(yml|yaml)'
"""

EXAMPLES = """
---
# Generate dynamic inventory of every device
plugin: cisco.aci.aci_inventory_system
host: 192.168.1.90
username: admin
password: PASSWORD
validate_certs: false

# (Optional) Generate inventory and put devices into groups based on role: spine, leaf, controller
keyed_groups:
  - prefix: role
    key: role

# (Optional) Generate inventory and use the compose variables to define how we want to connect
compose:
  ansible_connection: "'ansible.netcommon.httpapi'"
  ansible_network_os: "'cisco.aci.aci'"
  ansible_host: "'192.168.1.90'"
"""

import atexit
import time
import tempfile
import shutil
import typing as t
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import (
    RESERVED_ANSIBLE_INVENTORY_KEYS,
    MOCKED_CONSTRUCTED_INVENTORY_ARGUMENT_SPEC,
)
from ansible.module_utils.common.arg_spec import ArgumentSpecValidator
from ansible.module_utils.common.text.converters import to_native
from ansible.errors import AnsibleError
from ansible.utils.display import Display


display = Display()


class MockAnsibleModule(object):
    def __init__(self, argument_spec, parameters):
        """Mock AnsibleModule

        This is needed in order to use the aci methods which assume to be working
        with a module only.
        """

        self._socket_path = None
        self._debug = False
        self._diff = False
        self._tmpdir = None
        self.check_mode = False
        self.params = dict()

        validator = ArgumentSpecValidator(argument_spec)
        result = validator.validate(parameters)

        if result.error_messages:
            self.warn("Validation failed: {0}".format(", ".join(result.error_messages)))

        self.params = result.validated_parameters

    @property
    def tmpdir(self):
        if self._tmpdir is None:
            basefile = "ansible-moduletmp-%s-" % time.time()
            try:
                tmpdir = tempfile.mkdtemp(prefix=basefile)
            except (OSError, IOError) as e:
                self.fail_json(msg="Failed to create remote module tmp path with prefix %s: %s" % (basefile, to_native(e)))
            atexit.register(shutil.rmtree, tmpdir)
            self._tmpdir = tmpdir

        return self._tmpdir

    def warn(self, warning):
        display.vvv(warning)

    def fail_json(self, msg, **kwargs) -> t.NoReturn:
        raise AnsibleError(msg)


class InventoryModule(BaseInventoryPlugin, Constructable):

    NAME = "cisco.aci.aci_inventory_system"

    def verify_file(self, path):
        """return true/false if this is possibly a valid file for this plugin to consume"""
        valid = False
        if super(InventoryModule, self).verify_file(path):
            # base class verifies that file exists and is readable by current user
            if path.endswith(("cisco_aci.yaml", "cisco_aci.yml")):
                valid = True
        return valid

    def parse(self, inventory, loader, path, cache=True):

        # call base method to ensure properties are available for use with other helper methods
        super(InventoryModule, self).parse(inventory, loader, path, cache)

        # this method will parse 'common format' inventory sources and
        # update any options declared in DOCUMENTATION as needed
        config = self._read_config_data(path)

        argument_spec = aci_argument_spec()

        # Avoid argument spec validation warnings for plugin options
        # Options are retrieved from the constructed plugin docs, see below links for more information:
        #   - https://github.com/ansible/ansible/blob/devel/lib/ansible/plugins/inventory/constructed.py
        #   - https://github.com/ansible/ansible/blob/devel/lib/ansible/plugins/doc_fragments/constructed.py
        argument_spec.update(MOCKED_CONSTRUCTED_INVENTORY_ARGUMENT_SPEC)

        module = MockAnsibleModule(
            argument_spec=argument_spec,
            parameters=config,
        )

        aci = ACIModule(module)
        aci.construct_url(root_class=dict(aci_class="topSystem"))

        aci.get_existing()

        # parse data and create inventory objects:
        for device in aci.existing:
            attributes = device.get("topSystem", {}).get("attributes", {})
            if attributes.get("name"):
                self.add_host(
                    attributes.get("name"),
                    # replace the reserved Ansible inventory keys from host topSystem attributes
                    {RESERVED_ANSIBLE_INVENTORY_KEYS.get(key, key): value for key, value in attributes.items()},
                )

    def add_host(self, hostname, host_vars):
        self.inventory.add_host(hostname, group="all")

        if host_vars.get("oobMgmtAddr", "0.0.0.0") != "0.0.0.0":
            self.inventory.set_variable(hostname, "ansible_host", host_vars.get("oobMgmtAddr"))
        elif host_vars.get("inbMgmtAddr", "0.0.0.0") != "0.0.0.0":
            self.inventory.set_variable(hostname, "ansible_host", host_vars.get("inbMgmtAddr"))
        else:
            self.inventory.set_variable(hostname, "ansible_host", host_vars.get("address"))

        for var_name, var_value in host_vars.items():
            self.inventory.set_variable(hostname, var_name, var_value)

        strict = self.get_option("strict")

        # Add variables created by the user's Jinja2 expressions to the host
        self._set_composite_vars(self.get_option("compose"), host_vars, hostname, strict=True)

        # Create user-defined groups using variables and Jinja2 conditionals
        self._add_host_to_composed_groups(self.get_option("groups"), host_vars, hostname, strict=strict)
        self._add_host_to_keyed_groups(self.get_option("keyed_groups"), host_vars, hostname, strict=strict)
